// $Id: ProjectFileView.java 41 2010-04-03 20:04:12Z marcusvnac $
// Copyright (c) 2006 The Regents of the University of California. All
// Rights Reserved. Permission to use, copy, modify, and distribute this
// software and its documentation without fee, and without a written
// agreement is hereby granted, provided that the above copyright notice
// and this paragraph appear in all copies. This software program and
// documentation are copyrighted by The Regents of the University of
// California. The software program and documentation are supplied "AS
// IS", without any accompanying services from The Regents. The Regents
// does not warrant that the operation of the program will be
// uninterrupted or error-free. The end-user understands that the program
// was developed for research purposes and is advised not to rely
// exclusively on the program for any reason. IN NO EVENT SHALL THE
// UNIVERSITY OF CALIFORNIA BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT,
// SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS,
// ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF
// THE UNIVERSITY OF CALIFORNIA HAS BEEN ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE. THE UNIVERSITY OF CALIFORNIA SPECIFICALLY DISCLAIMS ANY
// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE SOFTWARE
// PROVIDED HEREUNDER IS ON AN "AS IS" BASIS, AND THE UNIVERSITY OF
// CALIFORNIA HAS NO OBLIGATIONS TO PROVIDE MAINTENANCE, SUPPORT,
// UPDATES, ENHANCEMENTS, OR MODIFICATIONS.

package org.argouml.persistence;

import java.io.File;

import javax.swing.Icon;
import javax.swing.filechooser.FileView;

import org.argouml.application.helpers.ResourceLoaderWrapper;

/**
 * Provides an icon for project files. 
 * 
 * @since Jun 2, 2006
 * @author andrea.nironi@gmail.com @stereotype singleton
 */
public final class ProjectFileView extends FileView {

    private static ProjectFileView instance = new ProjectFileView();
    
    /**
     * Constructor for ProjectFileView.
     */
    private ProjectFileView() {
        
    }

    /**
     * Returns the singleton instance.
     * 
     * @return ProjectFileView
     */
    public static ProjectFileView getInstance() {
        return instance;
    }
    
    /**
     * Load an icon for a supported project file.
     *  
     * @param   f the file to check 
     * @return  a nice icon if the file is known as a project file, 
     * 			otherwise a default one
     */
    public Icon getIcon(File f) {
    	AbstractFilePersister persister = PersistenceManager.getInstance()
				.getPersisterFromFileName(f.getName());
        if (persister != null && persister.hasAnIcon()) {
            return ResourceLoaderWrapper.lookupIconResource("UmlNotation");
        } else {
            return null;
        }
    } 
}
